@@ -166,6 +166,8 @@ class DataReader(_StateMixin):
         """
         pass  # pragma: no cover
 
+    read = load  # pragma: no cover
+
     def __call__(self, *args, **kwargs):
         """Alias for :meth:`self.load`."""
         return self.load(*args, **kwargs)
@@ -192,6 +194,8 @@ class DataWriter(_StateMixin):
         """
         pass  # pragma: no cover
 
+    write = save  # pragma: no cover
+
     def __call__(self, *args, **kwargs):
         """Alias for :meth:`self.save`."""
         return self.save(*args, **kwargs)
