@@ -1,16 +1,15 @@
-"""Abstract classes to wrap Keras model.
-
-T
-SegModel: Abstract wrapper for Keras model used for semantic segmentation.
-"""
-
 from abc import ABC, abstractmethod
 
 import numpy as np
 
-from dosma.data_io.med_volume import MedicalVolume
+from dosma.core.med_volume import MedicalVolume
 from dosma.defaults import preferences
 
+try:
+    import keras.backend as K
+except ImportError:  # pragma: no cover
+    pass
+
 
 class SegModel(ABC):
     """
@@ -60,6 +59,9 @@ class SegModel(ABC):
         """
         pass
 
+    def __call__(self, *args, **kwargs):
+        return self.generate_mask(*args, **kwargs)
+
     def __preprocess_volume__(self, volume: np.ndarray):
         """
         Preprocess volume prior to putting as input into segmentation network
@@ -82,9 +84,10 @@ class KerasSegModel(SegModel):
     Abstract wrapper for Keras model used for semantic segmentation
     """
 
-    def build_model(self, input_shape, weights_path):
+    def build_model(self, input_shape, weights_path=None):
         keras_model = self.__load_keras_model__(input_shape)
-        keras_model.load_weights(weights_path)
+        if weights_path:
+            keras_model.load_weights(weights_path)
 
         return keras_model
 
@@ -99,6 +102,9 @@ class KerasSegModel(SegModel):
         """
         pass
 
+    def __del__(self):
+        K.clear_session()
+
 
 # ============================ Preprocessing utils ============================
 __VOLUME_DIMENSIONS__ = 3
